// Note:
// For image mime types, it relies on the fact that Gecko will correctly
// display an image even if the server sent it with a mime type that does not
// match the real image type. For instance, a PNG image sent with
// image/jpeg Content-Type will still be displayed properly.

const MIMES_HTTP = [
  "text/plain",
  "text/html",
  "view-source",
  "text/xml",
  "image/jpeg",
  "other"
];

const MIMES_NON_HTTP = [
  "view-source"
];

const ADDITIONAL_MIMES_PREF = "extensions.openinbrowser.additional_mimes";

var OpenInBrowser = {

  get _netutil() {
    delete this._netutil;
    return this._netutil = Cc["@mozilla.org/network/util;1"].
                           getService(Ci.nsINetUtil);
  },

  get strings() {
    delete this.strings;
    return this.strings = document.getElementById("strings_openInBrowser");
  },

  populateMenu: function OIB_buildMenu(popup, uri) {
    var mimes;

    if (/^http/.test(uri.scheme)) {
      mimes = MIMES_HTTP;

      var additMimes = Application.prefs.getValue(ADDITIONAL_MIMES_PREF, null);
      if (additMimes) {
        mimes = mimes.slice();
        for each (let m in additMimes.split("|")) {
          mimes.push(m);
        }
      }
    } else {
      mimes = MIMES_NON_HTTP;
    }

    while (popup.firstChild)
      popup.removeChild(popup.firstChild);

    for each (let mime in mimes) {
      var item = document.createElement("menuitem");
      item.setAttribute("id", "mimeid_" + mime);

      var label = mime;
      // Use .stringBundle.GetStringFromName() instead of .getString()
      // to prevent dumping an exception if not found
      try {
        label = this.strings.stringBundle.GetStringFromName(mime + ".label");
      } catch(e) { }

      item.setAttribute("label", label);

      try {
        var accesskey = this.strings.stringBundle
                            .GetStringFromName(mime + ".accesskey");
        item.setAttribute("accesskey", accesskey);
      } catch(e) { }

      popup.appendChild(item);
    }
  },

  validateMime: function OIB_validateMime(mime) {
    var type = this._netutil.parseContentType(mime, {}, {});

    if (!type) {
      alert(this.strings.getFormattedString("InvalidMimeType", [mime]));
      return false;
    }
    return true;
  },

  reloadWithMime: function OIB_reloadWithMime(uri, mime) {
    if (mime == "view-source") {
      var iosvc = Cc["@mozilla.org/network/io-service;1"].
                  getService(Ci.nsIIOService);
      uri = iosvc.newURI("view-source:" + uri.spec, null, null);
    } else {
      var interceptor = Cc["@spasche.net/openinbrowser;1"].
                        getService().wrappedJSObject;
      interceptor.addInterceptInfo(uri.spec, mime);
    }

    Application.activeWindow.activeTab.load(uri);
  }
}